const WebSocket = require('ws');
const Message = require('./Message');

class Client {
    constructor(token) {
        this.token = token;
        this.clientUri = "https://connectx.unboundllc.tech/";
        this.events = {};
        this.clientid = null;
        this.websocket = null;
    }

    on(event, callback) {
        this.events[event] = this.events[event] || [];
        this.events[event].push(callback);
    }

    login(){
        this.websocket = new WebSocket(this.clientUri + "ws", {
            headers: {
              'Authorization': `Basic ${this.token}`
            }
        });

        this.websocket.onopen = () => {
            this.websocket.send("login");
            this.trigger("log" , "Connected to WebSocket Server Successfully");
        };

        this.websocket.onclose = () => {
            this.trigger("disconnect" , new Date())
        }

        this.websocket.onerror = (error) => {
            this.trigger("disconnect" , new Date());
        }

        this.websocket.onmessage = (event) => {
            let data = JSON.parse(event.data);
            let main_event = data.event;
            delete data.event;
            if(main_event == "ready"){
                this.clientid = data.authorId;
            }
            if(main_event == "message"){
                this.trigger(main_event , data.data)
            }else{
                this.trigger(main_event , {
                    ...data
                })
            }
        };
    }

    async servers() {
        const headers = new Headers();
        headers.append("Authorization", `Basic ${this.token}`);

        const requestOptions = {
            method: "GET",
            headers: headers,
            redirect: "follow"
        };

        let response = await fetch(`${this.clientUri}api/servers`, requestOptions);
        let result = await response.json();

        return result.servers;
    }

    /**
     * 
     * @param {Message} MessageData 
     */
    async send(MessageData) {
        const headers = new Headers();
        headers.append("Authorization", `Basic ${this.token}`);

        const formdata = new FormData();
        formdata.append("authorName", MessageData.authorName);
        formdata.append("authorIcon", MessageData.authorIcon);
        formdata.append("serverName", MessageData.serverName);
        formdata.append("serverIcon", MessageData.serverIcon);
        formdata.append("fields", MessageData.fields);
        formdata.append("message", MessageData.message);

        const requestOptions = {
            method: "POST",
            headers: headers,
            body: formdata,
            redirect: "follow"
        };

        let response = await fetch(`${this.clientUri}api/message`, requestOptions);
        let result = await response.json();

        return result.message;
    }

    trigger(event, data) {
        if (this.events[event]) {
            for (const callback of this.events[event]) {
                callback(data);
            }
        }
    }
}

module.exports = Client;