class Message {
    /**
     * @typedef {Object} MessageFields
     * @property {string|null} authorName
     * @property {URL|null} authorIcon
     * @property {string|null} serverName
     * @property {string|null} serverIcon
     * @property {string|null} message
     * @property {string|null} fields
     * @property {boolean|null} isSameId
     */

    constructor() {
        this.authorName = null;
        this.authorIcon = "auto";
        this.serverName = null;
        this.serverIcon = null;
        this.message = null;
        this.fields = [];
        this.isSameId = false;
    }

    /**
     * Fill Product fields with new values
     * @param {MessageFields} newFields - Object containing new values for Category fields
     */
    fill (newFields) {
        for (let field in newFields) {
            if (this.hasOwnProperty(field) && newFields.hasOwnProperty(field)) {
                if (this[field] !== 'undefined') {
                    this[field] = newFields[field];
                }
            }
        }

        return this.toArray();
    }

    toArray() {
        return {
            authorName: this.authorName,
            authorIcon: this.authorIcon,
            serverName: this.serverName,
            serverIcon: this.serverIcon,
            message: this.message,
            fields: this.fields,
            isSameId: this.isSameId
        };
    }

}

module.exports = Message;